/***************************************************************
 * proj1.cc ---- main program to test the information cluster  *
 *               for stack class (C++ version)                 *
 ***************************************************************/

#include <fstream.h> // ifstream
#include <stdio.h>  // NULL
#include "stack.hh"  // class Stack

extern "C" exit(int); // exit function

#define default_file "proj1.txt" // default file name used for input 

/* function prototypes */
void read_into_stack(ifstream fp, Stack &stack_id);
void write_onto_stack(Stack &stack_id1,Stack &stack_id2);
void error(char *s1, char *s2 = "");

// ---- <void error(char *s1, char *s2 = "")> ---- //
/* error handler */
void error(char *s1, char *s2 = "")
{
  cerr << s1 << ' ' << s2 << '\n';
  exit(1);
}

// <main program> //
/* test the information cluster for stack class
   tested member functions of class Stack:
   . stack constructor/destructor . stack copier
   . show all elements in a stack (from top to bottom) */
main(int argc, char *argv[])
{
  // open file specified on command line or the defaulted one
  ifstream fp((argc>1) ? argv[1] : default_file);
 
  // check i/o status
  if (!fp) {
    if (argc > 1) {
       cerr << "cannot open file: " << argv[1] << "\n";
       cout << "now try to open the default file: " << default_file << "\n";
       // open the default one
       ifstream gp;      // looks silly, but there is a problem on cat system
       gp.open(default_file); // this is a tempory solution (gp is not necessary)
       if (!gp) error("cannot open file: ",default_file);
       argc = 1;
       fp = gp;
     }
    else error("cannot open file: ", default_file);
  };
  
  cout << "This is file " << ((argc == 1) ? default_file : argv[1])
    << "\nProcessed by " <<  argv[0] << "\n";

  // Create stack1,stack3 (definition and construction go togather)
  Stack stack1,stack3;  

  // read the stream in file into stack1
  read_into_stack(fp,stack1);

  // copy stack1 to stack 3
  stack1.copy(stack3);

  // print a centered line 
  cout << "\n--------------------- Reversed Stream -----------------------\n";

  // show reversed stream in stack1
  stack1.show();

  // Create stack2,stack4 (definition and construction go togather)
  Stack stack2, stack4;  

  // write the reversed stream from stack1 onto stack2 
  write_onto_stack(stack1,stack2); 

  // print a centered line
  cout << "\n--------------------- Normal Stream -------------------------\n";

  // show  stream in stack2
  stack2.show();

  cout << '\n'; // end newline 

  // copy stack2 to stack4
  stack2.copy(stack4);  

 // show once more using copied stacks
  cout << "\n------------------ Again Reversed Stream --------------------\n";
  stack3.show();
  cout << "\n------------------ Again Normal Stream ----------------------\n";
  stack4.show();
  cout << '\n'; 

  // stacks will be destroyed automatically when progam exits
  exit(0); 
}

// ---- <void read_into_stack(ifstream fp, Stack &stack_id)> ---- //
// read stream from a file into stack //
inline void read_into_stack(ifstream fp, Stack &stack_id)
{
  char ch;
  while (fp.get(ch)) stack_id.push(ch);

  return;
}

// ---- <void write_onto_stack(Stack &stack_id1,Stack& stack_id2)> ---- //
// move elements from a stack into another stack //
inline void write_onto_stack(Stack &stack_id1,Stack& stack_id2)
{
  char ch;
  while ((ch = stack_id1.pop()) != NULL) stack_id2.push(ch); 

  return;
}

